from importlib.metadata import version

try:
    __version__ = version("dony")
except Exception:
    __version__ = "unknown"

from .command import command
from .shell import shell
from .prompts.autocomplete import autocomplete
from .prompts.confirm import confirm
from .prompts.input import input
from .prompts.path import path
from .prompts.press_any_key_to_continue import press_any_key_to_continue
from .prompts.select import select
from .prompts.print import print
from .prompts.error import error
from .run_dony.run_dony import run_dony
from .prompts.success import success
from .prompts.select_or_input import select_or_input



import re

import dony


@dony.command()
def pipx_install_dot():
    """Installs current branch to pipx"""

    dony.shell("pipx install . --force")


if __name__ == "__main__":
    pipx_install_dot()



from typing import Sequence, Union, Optional, Tuple
import subprocess
import questionary
from questionary import Choice


from dony import confirm


def select(
    message: str,
    choices: Sequence[Union[str, Tuple[str, str], Tuple[str, str, str]]],
    default: Optional[Union[str, Sequence[str]]] = None,
    multi: bool = False,
    fuzzy: bool = False,
    default_confirm: bool = False,
    provided_answer: str = None,
) -> Union[None, str, Sequence[str]]:
    """
    Prompt the user to select from a list of choices, each of which can have:
      - a display value
      - a short description (shown after the value)
      - a long description (shown in a right-hand sidebar in fuzzy mode)

    If fuzzy is True, uses fzf with a preview pane for the long descriptions.
    Falls back to questionary if fzf is not available or fuzzy is False.
    """

    # - Check if provided answer is set

    if provided_answer is not None:
        if provided_answer not in choices:
            raise ValueError(f"Provided answer '{provided_answer}' is not in choices.")
        return provided_answer

    # - If default is present and default_confirm is True, then ask for confirmation to just use default

    if default is not None and default_confirm:
        # - Ask for confirmation

        if confirm(message + f"\nUse default? [{default}]"):
            return default

    # Helper to unpack a choice tuple or treat a plain string
    def unpack(c):
        if isinstance(c, tuple):
            if len(c) == 3:
                return c  # (value, short_desc, long_desc)
            elif len(c) == 2:
                return (c[0], c[1], "")
            elif len(c) == 1:
                return (c[0], "", "")
        else:
            return (c, "", "")

    if fuzzy:
        try:
            delimiter = "\t"
            lines = []

            # Map from the displayed first field back to the real value
            display_map: dict[str, str] = {}

            for choice in choices:
                value, short_desc, long_desc = unpack(choice)
                display_map[value] = value
                lines.append(f"{value}{delimiter}{short_desc}{delimiter}{long_desc}")

            cmd = [
                "fzf",
                "--read0",  # ← treat NUL as item separator
                "--prompt",
                f"{message} 👆",
                "--with-nth",
                "1,2",
                "--delimiter",
                delimiter,
                "--preview",
                "echo {} | cut -f3",
                "--preview-window",
                "down:30%:wrap",
            ]

            proc = subprocess.Popen(
                cmd,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL,
                text=True,
            )
            output, _ = proc.communicate(input="\0".join(lines))
            if not output:
                raise KeyboardInterrupt

            # fzf returns lines like "disp1<sep>disp2", so split on the delimiter
            picked_disp1 = [
                line.split(delimiter, 1)[0] for line in output.strip().splitlines()
            ]
            results = [display_map[d] for d in picked_disp1]
            return results if multi else results[0]

        except FileNotFoundError:
            pass

    # Fallback to questionary
    q_choices = []

    for choice in choices:
        value, short_desc, long_desc = unpack(choice)

        if long_desc and short_desc:
            # suffix after the short description
            title = f"{value} - {short_desc} (description available)"
        elif long_desc and not short_desc:
            # no short_desc, suffix after the value
            title = f"{value} (description available)"
        elif short_desc:
            title = f"{value} - {short_desc}"
        else:
            title = value

        q_choices.append(
            Choice(
                title=title,
                value=value,
                checked=value in (default or []),
            )
        )

    if multi:
        result = questionary.checkbox(
            message=message,
            choices=q_choices,
            qmark="•",
            instruction="",
        ).ask()

        if result is None:
            raise KeyboardInterrupt

        return result

    result = questionary.select(
        message=message,
        choices=q_choices,
        default=default,
        qmark="•",
        instruction=" ",
    ).ask()

    if result is None:
        raise KeyboardInterrupt

    return result


def example():
    selected = select(
        "Give me that path",
        choices=[
            ("foo", "", "This is the long description for foo."),
            ("bar", "second option", "Detailed info about bar goes here."),
            ("baz", "third one", "Here’s a more in-depth explanation of baz."),
            ("qux", "", "Qux has no short description, only a long one."),
        ],
        # choices=['foo', 'bar', 'baz', 'qux'],
        multi=True,
        fuzzy=False,
        default=["foo"],
        default_confirm=True,
    )
    print(selected)


if __name__ == "__main__":
    example()



import questionary


def path(
    message: str,
    provided_answer: str = None,
):
    if provided_answer is not None:
        return provided_answer

    result = questionary.path(
        message=message,
        qmark="•",
    ).ask()

    if result is None:
        raise KeyboardInterrupt


def example():
    print(
        path(
            "Give me that path",
        )
    )


if __name__ == "__main__":
    example()



from typing import Sequence, Union, Optional, List

import questionary
from prompt_toolkit.styles import Style


def autocomplete(
    message: str,
    choices: List[str],
    default: Optional[str] = "",
    provided_answer: str = None,
):
    if provided_answer is not None:
        return provided_answer

    result = questionary.autocomplete(
        message=message,
        choices=choices,
        default=default,
        qmark="•",
        style=Style(
            [
                ("question", "fg:ansiblue"),  # the question text
            ]
        ),
    )

    if result is None:
        raise KeyboardInterrupt


def example():
    print(
        autocomplete(
            "Give me that path",
            choices=["foo", "bar"],
        ).ask()
    )


if __name__ == "__main__":
    example()



import questionary
from prompt_toolkit import print_formatted_text
from prompt_toolkit.formatted_text import FormattedText


def error(
    text: str,
    prefix: str = "❌ ",
):
    return print_formatted_text(
        FormattedText(
            [
                ("class:qmark", "• "),
                ("class:question", prefix + text),
            ]
        ),
        style=questionary.Style(
            [
                ("question", "fg:ansired"),  # the question text
                ("question", "bold"),  # the question text
            ]
        ),
    )


def example():
    error("Failed to do something important")


if __name__ == "__main__":
    example()



#!/usr/bin/env python3
import json
import os
import sys
from pathlib import Path
from textwrap import dedent

from dony import shell
from dony.get_dony_path import get_donyfiles_path
from dony.parse_unknown_args import parse_unknown_args


def main():
    # - Parse any input arguments (unknown for now)

    args = parse_unknown_args(sys.argv)

    # - Run version command

    if args["keyword"]:
        first_key, first_value = next(iter(args["keyword"].items()))

        if first_key == "version":
            assert first_value == [True]
            assert len(args["keyword"]) == 1, "version command takes no arguments"
            from dony import __version__

            print(f"dony version {__version__}")
            sys.exit(0)

        # - Run help command

        if first_key == "help":
            assert first_value == [True]
            assert len(args["keyword"]) == 1, "help command takes no arguments"
            print(
                dedent("""
                        dony: dony [OPTIONS] COMMAND [ARGS]
                
                        Options:
                          --version       Show version information and exit
                          --help          Show this help message and exit
                        
                        Commands:
                          my_command      Default operation
                        
                        Example:
                          dony my_command --arg_key arg_value
                       """)
            )
            sys.exit(0)

        # - Run 'init command'

        if first_key == "init":
            assert first_value == [True]
            assert len(args["keyword"]) == 1, "init command takes no arguments"

            # - Create donyfiles dir if it does not exist

            if not (Path.cwd() / "donyfiles").exists():
                os.mkdir(Path.cwd() / "donyfiles")

            # - Os into dony dir

            os.chdir(Path.cwd() / "donyfiles")

            # - Run uv init

            shell(
                'uv init --name dony-commands --description "dony environment"',
                working_directory=None,
            )

            # - Create environment

            shell(
                "uv sync",
                working_directory=None,
            )

            # - Add packages

            shell(
                """uv add dony""",
                working_directory=None,
            )

            # - Remove hello.py file

            os.remove("main.py")

            # - Create .gitignore file allowing uv files

            with open(".gitignore", "w") as f:
                f.write(
                    dedent("""
                            !.gitignore
                            !uv.lock
                            !pyproject.toml
                            !.python-version
                            !README.md
                            .venv
                    """).strip()
                )

            # - Create hello world example

            os.makedirs("commands/", exist_ok=True)

            with open("commands/hello_world.py", "w") as f:
                f.write(
                    dedent("""
                            import dony
                            
                            @dony.command()
                            def hello_world(name: str = "John"):
                                print(f"Hello, {name}!")
                            
                            """)
                )

            # - Add files to git

            shell(
                "git add .",
                working_directory=None,
            )

            sys.exit(0)

    # - Get dony dir

    root = Path.cwd()
    try:
        dony_path = get_donyfiles_path(root)

    except FileNotFoundError as e:
        print(e, file=sys.stderr)
        if len(args["positional"]) > 1:
            if args["positional"][1] == "help":
                print("Did you mean `dony --help`?")
            if args["positional"][1] == "init":
                print("Did you mean `dony --init`?")
            if args["positional"][1] == "version":
                print("Did you mean `dony --version`?")
        sys.exit(1)

    # - Run run_dony in local uv environment. Remove dony from the local directory as it shadows the dony module

    shell(
        """
        uv run --no-active python -c "import sys; sys.path.pop(0); import dony; from pathlib import Path; import sys; dony.run_dony(donyfiles_path=Path({}), args={})"

        """.format(
            # dony_dir / ".venv/bin/python",
            ('"' + str(dony_path) + '"').replace('"', '\\"'),
            json.dumps(args).replace('"', '\\"'),
        ),
        print_command=False,
        working_directory=dony_path,
    )


def example():
    import os

    import sys

    sys.argv = ["dony"]

    # os.chdir("../example/")
    os.chdir("/Users/marklidenberg/Documents/coding/repos/marklidenberg/jprint2")
    main()


if __name__ == "__main__":
    # main()
    example()



import os
from pathlib import Path
from typing import Union


def get_donyfiles_path(path: Union[str, Path] = ".") -> Path:
    # - Convert path to Path object

    if isinstance(path, str):
        path = Path(os.path.abspath(path))

    current_path = path

    while True:
        if (current_path / "donyfiles").exists():
            return current_path / "donyfiles"

        current_path = current_path.parent
        if current_path == current_path.parent:
            raise FileNotFoundError("Could not find 'donyfiles' directory")


def example():
    print(get_donyfiles_path(Path.cwd()))


if __name__ == "__main__":
    example()



import importlib.util
import inspect
import os
import sys
from collections import Counter, OrderedDict
from pathlib import Path
from dotenv import load_dotenv

from dony.prompts.error import error
from dony.prompts.select import select
from dony.run_dony.run_with_list_arguments import run_with_list_arguments


def run_dony(
    donyfiles_path: Path,
    args: OrderedDict = OrderedDict({}),
):
    # - Add dony root to path

    sys.path.append(str(donyfiles_path))

    # - Find all py files, extract all commands. If there is a file with filename not same as function name - rename it

    while True:
        file_paths = [
            p
            for p in donyfiles_path.rglob("commands/**/*.py")
            if not p.name.startswith("_")
        ]
        commands = {}  # {path: command}
        should_repeat = False

        for file_path in file_paths:
            # - Skip file if starts with _

            def _is_skipped(part):
                if part.startswith("_"):
                    return True
                if part == ".venv":
                    return True
                return False

            if any(_is_skipped(part) for part in str(file_path.absolute()).split("/")):
                continue

            # - Collect callables with attribute _dony_command == True

            def _load_module(path: Path):
                spec = importlib.util.spec_from_file_location(path.stem, path)
                module = importlib.util.module_from_spec(spec)

                spec.loader.exec_module(module)
                return module

            cmds = [
                member
                for _, member in inspect.getmembers(
                    _load_module(file_path), inspect.isfunction
                )
                if getattr(member, "_dony_command", False)
            ]

            # - Validate exactly one command in a file

            if len(cmds) != 1:
                print(
                    f"{file_path}: expected exactly one @command, found {len(cmds)}",
                    file=sys.stderr,
                )
                sys.exit(1)

            # - Rename file if it's name not the same as the function

            if file_path.stem != cmds[0].__name__:
                # - Repeat the cycle again since we will rename some files

                should_repeat = True

                # - Rename file

                os.rename(file_path, file_path.with_name(cmds[0].__name__ + ".py"))

                # - Git add if possible

                try:
                    os.system(
                        f"git add {file_path.with_name(cmds[0].__name__ + '.py')}"
                    )
                except:
                    print(
                        f"failed to add file to git: {file_path.with_name(cmds[0].__name__ + '.py')}"
                    )

            # - Validate command has _path

            commands[cmds[0]._path] = cmds[0]

        if not should_repeat:
            break

    # - Validate paths are unique

    counter = Counter(cmd._path for cmd in commands.values())
    duplicates = [path for path, count in counter.items() if count > 1]
    if duplicates:
        print(f"Duplicate commands: {duplicates}", file=sys.stderr)
        sys.exit(1)

    # - Return if no commands found

    if not commands:
        print("No commands found. Exiting...")
        sys.exit(0)

    # - Choose command and parse arguments

    if len(args["positional"]) == 1:  # no command was passed directly
        # - Interactive mode

        try:
            path = select(
                "Select command",
                choices=sorted(
                    [
                        (
                            ("📝 " if command.__doc__ else "️  ") + command._path,
                            "",
                            command.__doc__ or "",
                        )
                        for command in commands.values()
                    ],
                    key=lambda x: ("/" not in x[0], x[0].replace("📝 ", "").strip()),
                    reverse=True,
                ),
                fuzzy=True,
            )
        except KeyboardInterrupt:
            return error("Dony command interrupted")
    else:
        # - Command line mode

        path = args["positional"][1]

        # - Validate command exists

        if path not in commands:
            print(f"Unknown command: {path}", file=sys.stderr)
            print("\nAvailable commands:", file=sys.stderr)
            for cmd_path in sorted(commands.keys()):
                print(f"  {cmd_path}", file=sys.stderr)
            sys.exit(1)

    if not path:
        return

    print(f"️🍥 Running {path} from {donyfiles_path}...")

    # - Load dotenv from dony path and parent

    load_dotenv(dotenv_path=donyfiles_path / ".env")
    load_dotenv(dotenv_path=donyfiles_path.parent / ".env")

    # - Set dony path to env so that shell function can find it

    os.environ["_DONY_PATH"] = str(donyfiles_path)

    # - Run command with passed arguments

    run_with_list_arguments(
        func=commands[
            path.replace("📝 ", "").replace("\ufe0f", "").strip()
        ],  # 0xfe0f is an invisible "display this preceding character in emoji (color) style" character. Not sure why it's sometimes there
        list_kwargs=args["keyword"],
    )


if __name__ == "__main__":
    run_dony(
        donyfiles_path=Path(
            "/Users/marklidenberg/Documents/coding/repos/marklidenberg/dony"
        ),
        args=OrderedDict(positional=["hello_world"], keyword={}),
    )

    # import json
    #
    # run_dony(
    #     dony_dir=Path("../../example/dony"),
    #     args=json.loads('{"positional": ["hello_world"], "keyword": {"name": ["Mark"]}}'),
    # )



import os
from subprocess import CalledProcessError

import dony


@dony.command()
def hello_world():
    """Hello, world!"""
    dony.shell('echo "Hello, world!"')


if __name__ == "__main__":
    hello_world()



import questionary
from prompt_toolkit.styles import Style


def confirm(
    message: str,
    default: bool = True,
    provided_answer: str = None,
):
    """
    Prompt the user to confirm a decision.
    """

    # NOTE: typing is worse than using arrows, so we'll just use select instead of `questionary.confirm` with [Y/n]

    if provided_answer is not None:
        if provided_answer.lower() in ["y", "yes", "true", "1"]:
            return True
        elif provided_answer.lower() in ["n", "no", "false", "0"]:
            return False
        else:
            raise ValueError(
                f"Provided answer '{provided_answer}' is not a valid boolean value. Use one of 'y', 'yes', 'true', '1', 'n', 'no', 'false', '0'."
            )

    from dony.prompts.select import select  # avoid circular import

    result = (
        select(
            message=message,
            choices=["Yes", "No"] if default else ["No", "Yes"],
            multi=False,
            fuzzy=False,
        )
        == "Yes"
    )

    if result is None:
        raise KeyboardInterrupt

    return result


def example():
    print(confirm("Are you sure?"))


if __name__ == "__main__":
    example()



import itertools

from dony.shell import shell

import itertools
import sys
from collections import OrderedDict


def parse_unknown_args(arg_list: list) -> dict:
    """
    Turn a flat list like
      ['my_command',
       '--simple','simple',
       '--list1','a','b',
       '--list2','c','--list2','d']
    into
      {'positional': ['my_command'],
       'simple':     ['simple'],
       'list1':      ['a','b'],
       'list2':      ['c','d']}
    """
    left_tokens = iter(arg_list)
    result = {
        "positional": [],
        "keyword": OrderedDict(),
    }

    while True:
        # - Get next token

        try:
            token = next(left_tokens)
        except StopIteration:
            break

        # - Raise if short argument, not supported

        if token.startswith("-") and not token.startswith("--"):
            raise ValueError(f"Short arguments are not supported: {token}")

        # - Handle flag

        if token.startswith("--"):
            # - Init values

            values = []

            # - Strip --

            key = token.lstrip("-")  # --simple -> simple

            # - Consume _all_ subsequent non-flag tokens as values

            while True:
                try:
                    next_token = next(left_tokens)
                except StopIteration:
                    break
                if next_token.startswith("--"):
                    # push back this flag for the next iteration
                    left_tokens = itertools.chain([next_token], left_tokens)
                    break
                values.append(next_token)

            # - If no explicit values, treat as a boolean flag

            if not values:
                values = [True]

            # - Add values

            result["keyword"].setdefault(key, []).extend(values)

        else:
            # - Positional argument

            result["positional"].append(token)

    # - Return result

    return result


def example():
    import os

    shell("uv run python parse_unknown_args_example.py positional1 positional2 --simple simple --list1 a b --list2 c --list2 d")


if __name__ == "__main__":
    example()



from typing import Callable, Any, get_origin, List
import inspect


def run_with_list_arguments(func: Callable, list_kwargs: dict) -> Any:
    """
    Calls `func` by unpacking `list_kwargs`, where each value in `list_kwargs` is a list.
    - If the corresponding parameter annotation is a list (e.g. list[str]),
      the entire list is passed.
    - Otherwise, the first element of the list is passed.

    Example:
        def f(a: str, b: list[str]) -> str:
            return a + b[0]

        run_with_list_arguments(f, {"a": ["hello"], "b": ["world"]})

        # calls f(a="hello", b=["world"])
    """
    sig = inspect.signature(func)
    bound_args = {}

    for name, param in sig.parameters.items():
        if name not in list_kwargs:
            continue

        values = list_kwargs[name]
        if not isinstance(values, list):
            raise TypeError(
                f"Expected a list for argument '{name}', got {type(values).__name__}"
            )

        anno = param.annotation

        # Detect if the annotation is a list type
        if get_origin(anno) is list:
            # pass the full list
            bound_args[name] = values
        else:
            # pass only the first element
            if not values:
                raise ValueError(f"No values provided for argument '{name}'")
            bound_args[name] = values[0]

    return func(**bound_args)


def test():
    def f(a: str, b: List[str]) -> List[str]:
        return a, b

    assert run_with_list_arguments(f, {"a": ["hello"], "b": ["world"]}) == (
        "hello",
        ["world"],
    )


if __name__ == "__main__":
    test()



from pprint import pprint
from textwrap import dedent

import questionary
from prompt_toolkit import print_formatted_text
from prompt_toolkit.formatted_text import FormattedText


def print(
    text: str,
    line_prefix: str = "",
    color_style: str = "ansiblue",  # take colors from prompt_toolkit
):
    # - Dedent text

    text = dedent(text).strip()

    # - Add line prefix if needed

    if line_prefix:
        text = "\n".join([line_prefix + line for line in text.splitlines()])

    # - Print

    return print_formatted_text(
        FormattedText(
            [
                ("class:question", text),
            ]
        ),
        style=questionary.Style(
            [
                ("question", f"fg:{color_style}"),  # the question text
            ]
        ),
    )


def example():
    print("""echo "{"a": "b"}\nfoobar""", line_prefix="│ ")


if __name__ == "__main__":
    example()



from typing import List


from dony.prompts.select import select
from dony.prompts.input import input


def select_or_input(
    message: str,
    choices: List[str],
    allow_empty_string: bool = False,
    reject_choice: str = "✏️ Enter your own",
    provided_answer: str = None,
):
    if provided_answer is not None:
        return provided_answer

    result = select(
        message=message,
        choices=choices + [reject_choice],
    )

    if result != reject_choice:
        return result

    return input(
        message=message,
        allow_empty_string=allow_empty_string,
    )


def example():
    print(
        select_or_input(
            message="What is your name?",
            choices=["Alice", "Bob", "Charlie"],
        )
    )


if __name__ == "__main__":
    example()



import os.path
from pathlib import Path
from typing import Union

from dony.get_dony_path import get_donyfiles_path


def get_dony_root(current_path: Union[str, Path] = ".") -> Path:
    return get_donyfiles_path(current_path).parent


def example():
    print(get_dony_root())


if __name__ == "__main__":
    example()



import inspect
import os
import re
import sys
from pathlib import Path
from functools import wraps
from dataclasses import make_dataclass, fields, field
from typing import Any, get_origin, get_args

from dotenv import load_dotenv

from dony.shell import shell
from dony.prompts.error import error
from dony.get_dony_path import get_donyfiles_path
from dony.prompts.success import success


# deprecate lazy-dataclass-arguments
# def lazy_dataclass(cls):
#     """Lazy evaluation of dataclass attributes when they are accessed."""
#     import inspect
#     from dataclasses import dataclass
#
#     cls = dataclass(cls)
#
#     def __getattribute__(self, item):
#         value = object.__getattribute__(self, item)
#
#         # Only evaluate callables that look like zero-arg methods (self only)
#         if not item.startswith("_") and callable(value) and not inspect.ismethod(value):
#             if len(inspect.signature(value).parameters) == 0:
#                 evaluated = value()
#                 setattr(self, item, evaluated)
#                 return evaluated
#             elif len(inspect.signature(value).parameters) == 1 and "kwargs" in inspect.signature(value).parameters:
#                 evaluated = value(self.__dict__)
#                 setattr(self, item, evaluated)
#                 return evaluated
#             else:
#                 raise ValueError(f"Callable {value} can have 0 parameters of 1 parameter 'kwargs'")
#
#         return value
#
#     cls.__getattribute__ = __getattribute__
#     return cls


# end-deprecate


def command(path: str = None):
    """
    Decorator to mark a function as a dony command.
    - Builds a lazy dataclass of all parameters, allowing cascading defaults.
    - Executes callable defaults when their fields are accessed.
    - Ensures every parameter has a default value and the function name matches its filename.
    """

    def decorator(func):
        sig = inspect.signature(func)

        # - Validate that all parameters have default values

        for name, param in sig.parameters.items():
            if param.default is inspect._empty:
                raise ValueError(
                    f"Command '{func.__name__}': parameter '{name}' must have a default value"
                )

        # - Validate all parameters have string or List[str] types

        for name, param in sig.parameters.items():
            if not (
                param.annotation is str
                or get_origin(param.annotation) is list
                and get_args(param.annotation)[0] is str
            ):
                raise ValueError(
                    f"Command '{func.__name__}': parameter '{name}' must have a string or List[str] type"
                )

        # - Get file_path

        source_file = inspect.getsourcefile(func)
        if not source_file:
            raise RuntimeError(
                f"Could not locate source file for command '{func.__name__}'"
            )
        file_path = Path(source_file).resolve()

        # - Validate file_path is in donyfiles

        assert (
            "donyfiles" in file_path.parts
        ), f"Command '{func.__name__}' must be in 'donyfiles' directory"

        # - Validate filename matches function name

        # disabled for now, we rename file with running dony command

        # stem = file_path.stem
        # if func.__name__ != stem:
        #     raise ValueError(f"Command name '{func.__name__}' does not match filename '{stem}.py'")

        # - Compute or use provided path

        if path is None:
            parts = file_path.parts
            try:
                idx = parts.index("donyfiles")
            except ValueError:
                raise RuntimeError(
                    f"Cannot derive path: 'donyfiles' not found in '{file_path}'"
                )
            relative = Path(*parts[idx:]).as_posix()
            func._path = relative
        else:
            func._path = path

        # - Crop to last dony folder

        func._path = re.sub(r"^.*/donyfiles/", "", func._path).replace(".py", "")

        # crop commands prefixes
        if func._path.startswith("donyfiles/commands/"):
            func._path = func._path[len("donyfiles/commands/") :]
        if func._path.startswith("commands/"):
            func._path = func._path[len("commands/") :]

        func._dony_command = True

        # deprecate lazy-dataclass-arguments
        # # - Build a lazy dataclass for args
        #
        # field_defs = []
        #
        # for name, param in sorted(
        #     sig.parameters.items(), key=lambda pair: callable(pair[1].default)
        # ):
        #     ann = param.annotation if param.annotation is not inspect._empty else Any
        #
        #     if callable(param.default):
        #         # - Pass callables as is
        #
        #         field_defs.append((name, ann, param.default))
        #     else:
        #         # - Pass others as field with default value
        #
        #         field_defs.append(
        #             (name, ann, field(default_factory=lambda: param.default))
        #         )
        #
        # ArgsCls = lazy_dataclass(
        #     make_dataclass(f"{func.__name__.capitalize()}Args", field_defs)
        # )
        # end-deprecate

        @wraps(func)
        def wrapper(*args, **kwargs):
            # - Load dotenv in dony path or its parent

            if (
                os.path.basename(inspect.currentframe().f_back.f_code.co_filename)
                == "run_with_list_arguments.py"
            ):
                # running from command client
                donyfiles_path = get_donyfiles_path(
                    inspect.currentframe().f_back.f_back.f_back.f_code.co_filename
                )
            else:
                donyfiles_path = get_donyfiles_path(
                    inspect.currentframe().f_back.f_code.co_filename
                )

            load_dotenv(dotenv_path=donyfiles_path / ".env")
            load_dotenv(dotenv_path=donyfiles_path.parent / ".env")

            # - Bind partial to allow positional or keyword

            bound = sig.bind_partial(*args, **kwargs)
            bound.apply_defaults()

            # deprecate lazy-dataclass-arguments
            # # - Instantiate args object; fields will evaluate lazily
            #
            # args_obj = ArgsCls(**bound.arguments)
            #
            # #  -Force evaluation of all fields (cascading defaults)
            # final_kwargs = {}
            # for f in fields(ArgsCls):
            #     final_kwargs[f.name] = getattr(args_obj, f.name)
            # bounds.arguments = final_kwargs
            # end-deprecate

            # - Change directory to dony root

            os.chdir(donyfiles_path.parent)

            # - Call original function with resolved args

            try:
                result = func(**bound.arguments)
                success(f"Command '{func.__name__}' succeeded")
                return result
            except KeyboardInterrupt:
                return error("Dony command interrupted")

        # - Attach metadata to wrapper

        wrapper._dony_command = True
        wrapper._path = func._path
        return wrapper

    return decorator


def test():
    try:

        @command()
        def foo(
            a: str,
            b: str = "1",
            c: str = "2",
        ):
            return a + b + c

    except ValueError as e:
        assert str(e) == "Command 'foo': parameter 'a' must have a default value"

    try:

        @command()
        def bar(
            a: str = "0",
            b: str = "1",
            c: str = "2",
        ):
            return a + b + c
    except ValueError as e:
        assert str(e) == "Command name 'bar' does not match filename 'command.py'"


if __name__ == "__main__":
    test()



import questionary
from prompt_toolkit.styles import Style


def press_any_key_to_continue(message: str = "Press any key to continue..."):
    result = questionary.press_any_key_to_continue(
        message=message,
        style=Style(
            [
                ("question", "fg:ansiblue"),  # the question text
            ]
        ),
    ).ask()

    if result is None:
        raise KeyboardInterrupt


def example():
    print(press_any_key_to_continue())


if __name__ == "__main__":
    example()



from __future__ import annotations

import os.path
import subprocess
import sys
from inspect import currentframe
from pathlib import Path
from textwrap import dedent
from typing import Optional

from dony.prompts.error import error
from dony.prompts.print import print as dony_print
from dony.get_dony_root import get_dony_root
import pyperclip


class DonyShellError(Exception):
    pass


def shell(
    command: str,
    *,
    capture_output: bool = True,
    text: bool = True,
    exit_on_error: bool = True,
    error_on_unset: bool = True,
    echo_commands: bool = False,
    working_directory: Optional[str, Path] = None,
    quiet: bool = False,
    dry_run: bool = False,
    copy_dry_run_to_clipboard: bool = True,
    raise_on_error: bool = True,
    print_command: bool = True,
) -> Optional[str]:
    """
    Execute a shell command, streaming its output to stdout as it runs,
    and automatically applying 'set -e', 'set -u' and/or 'set -x' as requested.

    Args:
        command: The command line string to execute.
        capture_output: If True, captures and returns the full combined stdout+stderr;
                        if False, prints only and returns None.
        text: If True, treats stdout/stderr as text (str); if False, returns bytes.
        exit_on_error: If True, prepend 'set -e' (exit on any error).
        error_on_unset: If True, prepend 'set -u' (error on unset variables).
        echo_commands: If True, prepend 'set -x' (echo commands before executing).
        working_directory: If provided, change the working directory before executing the command.
        quiet: If True, suppresses output.
        dry_run: If True, prints the command without executing it.
        raise_on_error: If True, raises an exception if the command exits with a non-zero status.
        copy_dry_run_to_clipboard: If True, copies the dry run command to the clipboard.
        print_command: If True, prints the command before executing it.

    Returns:
        The full command output as a string (or bytes if text=False), or None if capture_output=False.

    Raises:
        subprocess.CalledProcessError: If the command exits with a non-zero status.
    """

    # - Get formatted command if needed

    if print_command or dry_run:
        try:
            formatted_command = shell(
                f"""
                    shfmt << 'EOF'
                    {command}
                """,
                quiet=True,
                print_command=False,
            )
        except Exception:
            formatted_command = command
    else:
        formatted_command = command

    # - Process dry_run

    if dry_run:
        dony_print(
            "🐚 Dry run\n" + formatted_command,
            color_style="ansipurple",
            # line_prefix="    ",
        )

        # - Copy to clipboard

        if copy_dry_run_to_clipboard:
            try:
                pyperclip.copy(formatted_command)
            except:
                error("Failed to copy dry-run to clipboard")

        return ""

    # - Print command

    if print_command and not quiet:
        dony_print(
            "🐚\n" + formatted_command,
            color_style="ansipurple",
            # line_prefix="    ",
        )

    # - Convert working_directory to string

    if isinstance(working_directory, Path):
        working_directory = str(working_directory)

    # - If relative - concat working directory with dony root

    if isinstance(working_directory, str) and not os.path.isabs(working_directory):
        working_directory = get_dony_root() / working_directory

    # - Build the `set` prefix from the enabled flags

    flags = "".join(
        flag
        for flag, enabled in (
            ("e", exit_on_error),
            ("u", error_on_unset),
            ("x", echo_commands),
        )
        if enabled
    )
    prefix = f"set -{flags}; " if flags else ""

    # - Dedent and combine the command

    full_cmd = prefix + dedent(command.strip())

    # - Execute with optional working directory

    proc = subprocess.Popen(
        full_cmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=text,
        cwd=working_directory,
    )

    # - Capture output

    buffer = []
    assert proc.stdout is not None
    while True:
        try:
            for line in proc.stdout:
                if not quiet:
                    print(line, end="")
                if capture_output:
                    buffer.append(line)
            break
        except UnicodeDecodeError:
            error("Error decoding output. Skipping the line")

    proc.stdout.close()
    return_code = proc.wait()

    output = "".join(buffer) if capture_output else None

    # - Raise if exit code is non-zero

    if return_code != 0 and raise_on_error:
        if "KeyboardInterrupt" in output:
            raise KeyboardInterrupt
        raise DonyShellError("Dony command failed")

    # - Print closing message

    if print_command and not quiet:
        dony_print(
            "—" * 80,
            color_style="ansipurple",
        )

    # - Return output

    return output.strip()


def example():
    # Default: set -eux is applied
    print(shell('echo "{"a": "b"}"'))

    # Disable only echoing of commands
    print(
        shell(
            'echo "no x prefix here"',
            echo_commands=False,
        )
    )

    # Run in a different directory
    output = shell("ls", working_directory="/tmp")
    print("Contents of /tmp:", output)

    try:
        shell('echo "this will fail" && false')
        raise Exception("Should have failed")
    except DonyShellError:
        pass


if __name__ == "__main__":
    example()



import questionary
from prompt_toolkit.styles import Style


def input(
    message: str,
    default: str = "",
    allow_empty_string: bool = False,
    provided_answer: str = None,
):
    if provided_answer is not None:
        return provided_answer

    while True:
        result = questionary.text(
            message,
            default=default,
            qmark="•",
            style=Style(
                [
                    ("question", "fg:ansiblue"),  # the question text
                ]
            ),
        ).ask()

        if result is None:
            raise KeyboardInterrupt

        if allow_empty_string or result:
            return result


def example():
    print(input(message="What is your name?"))


if __name__ == "__main__":
    example()
